<?php
/**
 * @file
 * Administration pages for managing promo block content.
 */

/**
 * Form definition for List admin page.
 */
function node_promo_picker_blocks_admin_list_form($state) {
  $form = array();
  // Get defined taxonomies.
  $tids = variable_get('promo_picker_taxonomy_settings', array());
  if (empty($tids)) {
    drupal_set_message(t('No taxonomies defined'), 'warning');
  }
  // Build table content.
  $header = array(
    'Taxonomy',
    array(
      'data' => 'Primary block',
      'colspan' => 2,
    ),
    array(
      'data' => 'Secondary block',
      'colspan' => 2,
    ),
  );
  foreach ($tids as $vid => $tid) {
    $vocab = db_result(db_query('Select name FROM {vocabulary} WHERE vid = %d', $vid));
    $form['vocabulary_' . $vid] = array(
      '#type' => 'fieldset',
      '#title' => check_plain('Vocabulary: ' . $vocab),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $rows = array();
    $allowed_elements = array(
      'table',
      'tr',
      'td',
      'th',
      'thead',
      'tbody',
      'a',
      'p',
      'div',
      'strong',
    );
    foreach ($tid as $t) {
      $taxonomy = db_result(db_query('SELECT name FROM {term_data} WHERE tid = %d', $t));
      $rows[] = array(
        $taxonomy,
        l(t('edit'), 'admin/build/promo_picker/blocks/edit/' . $t . '/primary'),
        l(t('reset'), 'admin/build/promo_picker/blocks/reset/' . $t . '/primary'),
        l(t('edit'), 'admin/build/promo_picker/blocks/edit/' . $t . '/secondary'),
        l(t('reset'), 'admin/build/promo_picker/blocks/reset/' . $t . '/secondary'),
      );
    }
    $table = theme('table', $header, $rows);
    $form['vocabulary_' . $vid]['table'] = array(
      '#type' => 'markup',
      '#value' => filter_xss($table, $allowed_elements),
    );
  }
  return $form;
}

/**
 * Reset promo block teasers.
 */
function node_promo_picker_blocks_admin_reset() {
  $tid = arg(5);
  $type = arg(6);
  // Set field to update.
  $field = ($type == 'primary') ? 'primary_flag' : 'secondary_flag';
  $alt_field = ($type == 'primary') ? 'secondary_flag' : 'primary_flag';
  // Reset teasers.
  $sql = 'UPDATE {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id SET ppt.`%s` = 0 WHERE pptt.tid = %d';
  db_query($sql, $field, $tid);
  $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE pptt.tid = %d AND ppt.`%s` = 0';
  db_query($sql, $tid, $alt_field);
  drupal_set_message(t('Promo block reset.'));
  // Set page to return to.
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/build/promo_picker/blocks';
  drupal_goto($goto);
}

/**
 * Form definition for editing promo block.
 *
 * @see node_promo_picker_blocks_admin_edit_form_validate()
 * @see node_promo_picker_blocks_admin_edit_form_submit()
 */
function node_promo_picker_blocks_admin_edit_form($state) {
  $form = array();
  // Get parameters.
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  $order_field = ($block == 'primary') ? 'primary_delta' : 'secondary_delta';
  // Taxonomy block details.
  $form['block_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Promo block details',
  );
  $form['block_panel']['tid'] = array(
    '#type' => 'item',
    '#title' => 'Taxonomy ID',
    '#value' => $tid,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $tname = db_result(db_query('SELECT name FROM {term_data} WHERE tid = %d', $tid));
  $form['block_panel']['taxonomy'] = array(
    '#type' => 'item',
    '#title' => 'Taxonomy',
    '#value' => $tname,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['block_panel']['placement'] = array(
    '#type' => 'item',
    '#title' => 'Type',
    '#value' => ucwords($block),
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $themes = array('' => 'Default (' . ucwords($block) . ' block)');
  $additional_themes = variable_get('promo_picker_additional_themes', array());
  foreach ($additional_themes as $f => $at) {
    $themes[$f] = $at['title'];
  }
  $theme = variable_get('promo_picker_theme_' . $tid . '_' . $block, '');
  $theme = (array_key_exists($theme, $themes)) ? $theme : '';
  $form['block_panel']['theme'] = array(
    '#type' => 'select',
    '#title' => 'Theme',
    '#options' => $themes,
    '#default_value' => $theme,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  // Add fields to add node to block.
  $form['add_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Add node teaser',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['add_panel']['node'] = array(
    '#type' => 'textfield',
    '#title' => 'Search nodes',
    '#description' => 'Search for node to add teaser for.',
    '#autocomplete_path' => 'admin/build/promo_picker/node/autocomplete',
  );
  $form['add_panel']['add_teasers'] = array(
    '#type' => 'submit',
    '#value' => 'Add node',
    '#validate' => array('node_promo_picker_add_node_to_block_validate'),
    '#submit' => array('node_promo_picker_add_node_to_block_submit'),
  );
  // Get all nodes displayed on the taxonomy.
  $form['teasers'] = array(
    '#tree' => TRUE,
    '#theme' => 'promo_block_teaser_list',
  );
  $sql = 'SELECT n.title AS node_title, n.nid, ppt.*, pptt.' . check_plain($order_field) . ' AS weight FROM {node} AS n JOIN {promo_picker_teasers} AS ppt ON n.nid = ppt.related_nid JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = %d AND ' . check_plain($field) . ' = 1 ORDER BY pptt.' . check_plain($order_field) . ' ASC';
  $result = db_query($sql, $tid);
  $max_teaser = variable_get('promo_picker_max_' . $block . '_teasers', 3);
  $has_teasers = FALSE;
  while ($n = db_fetch_object($result)) {
    // Get all teaser records for current node.
    $other = db_query('SELECT title, teaser_id FROM {promo_picker_teasers} WHERE related_nid = %d', $n->related_nid);
    $options = array();
    while ($t = db_fetch_object($other)) {
      $options[$t->teaser_id] = $t->title;
    }
    $query = array(
      'return_to' => 'admin/build/promo_picker/blocks/edit/' . $tid . '/' . $block,
      'referer' => ((!empty($_GET['return_to'])) ? $_GET['return_to'] : ''),
    );
    $form['teasers'][$n->teaser_id]['#teaser'] = (array) $n;
    $form['teasers'][$n->teaser_id]['node'] = array(
      '#value' => $n->node_title . ' (' . $n->nid . ')',
    );
    $form['teasers'][$n->teaser_id]['teaser_select'] = array(
      '#type' => 'select',
      '#options' => $options,
      '#default_value' => $n->teaser_id,
    );
    $form['teasers'][$n->teaser_id]['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $n->weight,
      '#delta' => $max_teaser,
    );
    $form['teasers'][$n->teaser_id]['edit'] = array(
      '#value' => l(t('Edit'),
        'admin/build/promo_picker/edit-teaser/' . $n->teaser_id,
        array(
          'query' => $query,
          'attributes' => array(
            'title' => 'Edit teaser',
          ),
        )
      )
    );
    $form['teasers'][$n->teaser_id]['new'] = array(
      '#value' => l(t('New'),
        'admin/build/promo_picker/create-teaser/' . $n->related_nid,
        array(
          'query' => $query,
          'attributes' => array(
            'title' => 'Create new teaser for node',
          ),
        )
      )
    );
    $form['teasers'][$n->teaser_id]['remove'] = array(
      '#value' => l(t('Remove'),
        'admin/build/promo_picker/blocks/remove-teaser/' . $n->teaser_id . '/' . $tid . '/' . $block,
        array(
          'query' => $query,
          'attributes' => array(
            'title' => 'Remove teaser from promo block',
          ),
        )
      )
    );
    $has_teasers = TRUE;
  }
  if ($has_teasers) {
    $form['save'] = array(
      '#type' => 'submit',
      '#value' => t('Save changes'),
    );
  }
  return $form;
}

/**
 * Validate add node form submission.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_add_node_to_block_validate($form, &$state) {
  // Get parameters.
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  // Validate node field.
  $nid = 0;
  if (preg_match('/\([0-9]+\)$/', $state['values']['node'], $match)) {
    $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
    $exists = db_result(db_query('SELECT nid FROM {node} WHERE nid = %d', $nid));
    if (!$exists) {
      form_set_error('node', 'Invalid node selected.');
    }
  }
  else {
    form_set_error('node', 'No node selected.');
  }
  // Check node can be added.
  $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE ppt.related_nid = %d AND pptt.tid = %d AND ppt.%s = 1';
  $exists = db_result(db_query($sql, $nid, $tid, $field));
  if ($exists) {
    form_set_error('node', 'Node already added to promo block.');
  }
}

/**
 * Process add node form submission.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_add_node_to_block_submit($form, &$state) {
  // Get parameters.
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  preg_match('/\([0-9]+\)$/', $state['values']['node'], $match);
  $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
  // Check if the node has a teaser and get the latest one.
  $teaser_id = db_result(db_query('SELECT MAX(teaser_id) FROM {promo_picker_teasers} WHERE related_nid = %d', $nid));
  // Up date teaser record or create new one.
  if ($teaser_id) {
    $teaser = db_fetch_object(db_query('SELECT * FROM {promo_picker_teasers} WHERE teaser_id = %d', $teaser_id));
    $teaser->{$field} = 1;
    drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
    $tax = new stdClass();
    $tax->teaser_id = $teaser->teaser_id;
    $tax->tid = $tid;
    drupal_write_record('promo_picker_teasers_taxonomy', $tax);
  }
  else {
    $node = db_fetch_object(db_query('SELECT n.*, r.teaser FROM {node} AS n JOIN {node_revisions} AS r ON n.vid = r.vid WHERE n.nid = %d', $nid));
    $teaser = new stdClass();
    $teaser->title = $node->title;
    $teaser->body = $node->teaser;
    $teaser->related_nid = $nid;
    $teaser->{$field} = 1;
    $teaser->uid = $GLOBALS['user']->uid;
    drupal_write_record('promo_picker_teasers', $teaser);
    $tax = new stdClass();
    $tax->teaser_id = $teaser->teaser_id;
    $tax->tid = $tid;
    drupal_write_record('promo_picker_teasers_taxonomy', $tax);
  }
  $query = (!empty($_GET['return_to'])) ? 'return_to=' . $_GET['return_to'] : '';
  drupal_goto('admin/build/promo_picker/blocks/edit/' . $tid . '/' . $block, $query);
}

/**
 * Validate promo block edit form.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_blocks_admin_edit_form_validate($form, &$state) {
  // Nothing to validate.
}

/**
 * Process promo block edit form submission.
 *
 * @see node_promo_picker_blocks_admin_edit_form()
 */
function node_promo_picker_blocks_admin_edit_form_submit($form, &$state) {
  $tid = arg(5);
  $block = arg(6);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  foreach ($state['values']['teasers'] as $teaser_id => $data) {
    // Change teaser record for taxonomy.
    if ($teaser_id != $data['teaser_select']) {
      db_query('DELETE FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = %d AND tid = %d', $teaser_id, $tid);
      $teaser = db_fetch_object(db_query('SELECT * FROM {promo_picker_teasers} WHERE teaser_id = %d', $data['teaser_select']));
      $teaser->{$field} = 1;
      drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
      $tax = new stdClass();
      $tax->teaser_id = $teaser->teaser_id;
      $tax->tid = $tid;
      drupal_write_record('promo_picker_teasers_taxonomy', $tax);
      $teaser_id = $teaser->teaser_id;
    }
    // Update teaser weight in promo block.
    $delta_field = ($block == 'primary') ? 'primary_delta' : 'secondary_delta';
    $taxonomy = db_fetch_object(db_query('SELECT * FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = %d AND tid = %d', $teaser_id, $tid));
    $taxonomy->{$delta_field} = $data['weight'];
    drupal_write_record('promo_picker_teasers_taxonomy', $taxonomy, array('teaser_id', 'tid'));
  }
  // Update block theme.
  if (!empty($state['values']['theme'])) {
    variable_set('promo_picker_theme_' . $tid . '_' . $block, $state['values']['theme']);
  }
  else {
    variable_del('promo_picker_theme_' . $tid . '_' . $block);
  }
  drupal_set_message(t('Promo block details saved.'));
  // Set page to return to.
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/build/promo_picker/blocks';
  drupal_goto($goto);
}

/**
 * Remove teaser record from promo block.
 */
function node_promo_picker_block_remove_teaser() {
  $teaser = arg(5);
  $tid = arg(6);
  $block = arg(7);
  $field = ($block == 'primary') ? 'primary_flag' : 'secondary_flag';
  $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.teaser_id = %d AND pptt.tid = %d AND ppt.%s = 1';
  db_query($sql, $teaser, $tid, $field);
  drupal_set_message(t('Teaser removed from block.'));
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/build/promo_picker/blocks/edit/' . $tid . '/' . $block;
  $query = (!empty($_GET['referer'])) ? 'return_to=' . $_GET['referer'] : '';
  drupal_goto($goto, $query);
}

/**
 * Theme teaser list into draggable table.
 */
function theme_promo_block_teaser_list($element) {
  $rows = array();
  foreach (element_children($element) as $key) {
    if (isset($element[$key]['node'])) {
      $teaser = &$element[$key];
      $row = array();
      $row[] = drupal_render($teaser['node']);
      $row[] = drupal_render($teaser['teaser_select']);
      $teaser['weight']['#attributes']['class'] = 'block-teaser-weight';
      $row[] = drupal_render($teaser['weight']);
      $row[] = drupal_render($teaser['edit']);
      $row[] = drupal_render($teaser['new']);
      $row[] = drupal_render($teaser['remove']);
      $rows[] = array('data' => $row, 'class' => 'draggable');
    }
  }
  if (empty($rows)) {
    $rows[] = array(array('data' => t('No teasers available.'), 'colspan' => '6'));
  }
  $header = array('Node', 'Teaser');
  $header[] = 'Weight';
  drupal_add_tabledrag('block-teaser-list', 'order', 'sibling', 'block-teaser-weight');
  $header[] = array('data' => 'Operations', 'colspan' => '3');
  return theme('table', $header, $rows, array('id' => 'block-teaser-list'));
}
