<?php
/**
 * @file
 * Administration pages for managing teaser records.
 */

/**
 * Form definition for teaser list screen.
 *
 * @see node_promo_picker_teasers_admin_list_form_submit()
 */
function node_promo_picker_teasers_admin_list_form($state) {
  $form = array();
  $form['search_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Filter list',
    '#collapsible' => TRUE,
    '#collapsed' => !variable_get('promo_picker_list_filtered_' . $GLOBALS['user']->uid, FALSE),
  );
  $form['search_panel']['node'] = array(
    '#type' => 'textfield',
    '#title' => 'Node',
    '#default_value' => variable_get('promo_picker_node_filter_' . $GLOBALS['user']->uid, ''),
  );
  $form['search_panel']['block'] = array(
    '#type' => 'select',
    '#title' => 'Promo block',
    '#options' => array(
      '' => 'All',
      'primary' => 'Primary',
      'secondary' => 'Secondary',
    ),
    '#default_value' => variable_get('promo_picker_block_filter_' . $GLOBALS['user']->uid, ''),
  );
  $form['search_panel']['buttons']['refine'] = array(
    '#type' => 'submit',
    '#value' => 'Refine',
  );
  $form['search_panel']['buttons']['reset'] = array(
    '#type' => 'submit',
    '#value' => 'Reset',
  );
  $args = node_promo_picker_get_teaser_filter_args();
  $sql = 'SELECT t.teaser_id, t.title, n.nid, n.title AS node_title, t.primary_flag, t.secondary_flag, u.name FROM {promo_picker_teasers} AS t JOIN {node} AS n ON t.related_nid = n.nid JOIN {users} AS u ON t.uid = u.uid ' . $args['join'] . ' ' . $args['where'] . ' ORDER BY t.teaser_id DESC';
  $result = pager_query(db_rewrite_sql($sql), 50, 0, NULL, $args['args']);
  $headers = array(
    'Teaser',
    'Node',
    'Primary',
    'Secondary',
    'User',
    array(
      'data' => 'Operations',
      'colspan' => 2,
    ),
  );
  $rows = array();
  while ($row = db_fetch_object($result)) {
    $rows[] = array(
      $row->title,
      $row->node_title . ' (NID: ' . $row->nid . ')',
      (($row->primary_flag) ? 'Yes' : 'No'),
      (($row->secondary_flag) ? 'Yes' : 'No'),
      $row->name,
      l(t('Edit'), 'admin/build/promo_picker/edit-teaser/' . $row->teaser_id, array('query' => array('return_to' => 'admin/build/promo_picker/teasers'))),
      l(t('Delete'), 'admin/build/promo_picker/delete-teaser/' . $row->teaser_id, array('query' => array('return_to' => 'admin/build/promo_picker/teasers'))),
    );
  }
  $form['table'] = array(
    'rows' => array('#value' => $rows),
    'header' => array('#value' => $headers),
    '#theme' => 'teaser_list_table',
    'pager' => array('#value' => theme('pager', NULL, 50, 0)),
  );
  return $form;
}

/**
 * Process submission of search filter form
 *
 * @see node_promo_picker_teasers_admin_list_form()
 */
function node_promo_picker_teasers_admin_list_form_submit($form, &$state) {
  variable_del('promo_picker_list_filtered_' . $GLOBALS['user']->uid);
  if ($state['values']['op'] == 'Reset') {
    unset($state['values']['node']);
    unset($state['values']['taxonomy']);
    unset($state['values']['block']);
  }
  if (!empty($state['values']['node'])) {
    variable_set('promo_picker_node_filter_' . $GLOBALS['user']->uid, $state['values']['node']);
    variable_set('promo_picker_list_filtered_' . $GLOBALS['user']->uid, TRUE);
  }
  else {
    variable_del('promo_picker_node_filter_' . $GLOBALS['user']->uid);
  }
  if (!empty($state['values']['block'])) {
    variable_set('promo_picker_block_filter_' . $GLOBALS['user']->uid, $state['values']['block']);
    variable_set('promo_picker_list_filtered_' . $GLOBALS['user']->uid, TRUE);
  }
  else {
    variable_del('promo_picker_block_filter_' . $GLOBALS['user']->uid);
  }
}

/**
 * Delete teaser record.
 */
function node_promo_picker_teasers_admin_delete() {
  $teaser = arg(4);
  db_query('DELETE FROM {promo_picker_teasers} WHERE teaser_id = %d', $teaser);
  db_query('DELETE FROM {promo_picker_teasers_images} WHERE teaser_id = %d', $teaser);
  db_query('DELETE FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = %d', $teaser);
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/build/promo_picker/teasers';
  drupal_set_message(t('Teaser record deleted.'));
  drupal_goto($goto);
}

/**
 * Get search filter arguments for teaser list.
 *
 * @return array
 *   defining where, join and arguments for sql
 */
function node_promo_picker_get_teaser_filter_args() {
  $where = $args = array();
  $join = '';
  // Check node filter.
  $node_filter = variable_get('promo_picker_node_filter_' . $GLOBALS['user']->uid, '');
  if (!empty($node_filter)) {
    $where[] = '(n.title LIKE \'%s\' OR n.nid = %d)';
    $args[] = '%' . $node_filter . '%';
    $args[] = $node_filter;
  }
  // Check block filter.
  $block_filter = variable_get('promo_picker_block_filter_' . $GLOBALS['user']->uid, '');
  if (!empty($block_filter)) {
    if ($block_filter == 'primary') {
      $where[] = 't.primary_flag = 1';
    }
    elseif ($block_filter == 'secondary') {
      $where[] = 't.secondary_flag = 1';
    }
  }
  // Set return array.
  $where = count($where) ? 'WHERE ' . implode(' AND ', $where) : '';
  return array('where' => $where, 'join' => $join, 'args' => $args);
}

/**
 * Form definition for add new node teaser.
 *
 * @see node_promo_picker_teasers_admin_add_form_validate()
 * @see node_promo_picker_teasers_admin_add_form_submit()
 */
function node_promo_picker_teasers_admin_add_form($state) {
  $form = array();
  // Node autocomplete search field.
  $form['node'] = array(
    '#type' => 'textfield',
    '#title' => 'Node',
    '#description' => 'Search for node to create teaser for.',
    '#autocomplete_path' => 'admin/build/promo_picker/node/autocomplete',
  );
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Create teaser'),
  );
  return $form;
}

/**
 * Validate add teaser form submission.
 *
 * @see node_promo_picker_teasers_admin_add_form()
 */
function node_promo_picker_teasers_admin_add_form_validate($form, &$state) {
  if (preg_match('/\([0-9]+\)$/', $state['values']['node'], $match)) {
    $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
    $exists = db_result(db_query('SELECT nid FROM {node} WHERE nid = %d', $nid));
    if (!$exists) {
      form_set_error('node', 'Invalid node selected.');
    }
  }
  else {
    form_set_error('node', 'No node selected to create teaser for.');
  }
}

/**
 * Process add teaser form submission.
 *
 * @see node_promo_picker_teasers_admin_add_form()
 */
function node_promo_picker_teasers_admin_add_form_submit($form, &$state) {
  if (preg_match('/\([0-9]+\)$/', $state['values']['node'], $match)) {
    $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
    drupal_goto('admin/build/promo_picker/create-teaser/' . $nid, 'return_to=admin/build/promo_picker/add-teaser');
  }
}

/**
 * Form definition for create a new teaser record.
 *
 * @see node_promo_picker_teasers_admin_create_form_validate()
 * @see node_promo_picker_teasers_admin_create_form_submit()
 */
function node_promo_picker_teasers_admin_create_form($state) {
  $form = array();
  // Get arguments from url.
  $nid = arg(4);
  $arg_tid = arg(5);
  $arg_block = arg(6);
  // Get node title.
  $title = db_result(db_query('SELECT title FROM {node} WHERE nid = %d', $nid));
  // Add teaser fields.
  $form['teaser_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Teaser',
  );
  $form['teaser_panel']['title'] = array(
    '#type' => 'textfield',
    '#title' => 'Title',
    '#default_value' => $title,
  );
  $form['teaser_panel']['teaser'] = array(
    '#type' => 'textarea',
    '#title' => 'Teaser',
  );
  $form['teaser_panel']['block'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Block',
    '#description' => 'Specify promo block to display teaser in.',
    '#options' => array('primary' => 'Primary', 'secondary' => 'Secondary'),
    '#default_value' => ((!empty($arg_block)) ? array($arg_block) : array()),
  );
  $image_node = variable_get('promo_picker_image_node', NULL);
  if (!empty($image_node)) {
    // Add image fields.
    $form['image_panel'] = array(
      '#type' => 'fieldset',
      '#title' => 'Images',
    );
    $form['image_panel']['images'] = array(
      '#tree' => TRUE,
      '#prefix' => '<div id="teaser-images">',
      '#suffix' => '</div>',
      '#theme' => 'form_teaser_images',
    );
    $num_images = variable_get('promo_picker_max_images', 3);
    if ($num_images == 0) {
      $form['image_panel']['images'][] = array(
        'image' => array(
          '#type' => 'textfield',
          '#title' => '',
          '#autocomplete_path' => 'admin/build/promo_picker/image/autocomplete',
          '#default_value' => '',
        ),
        'weight' => array(
          '#type' => 'weight',
          '#default_value' => 0,
          '#delta' => 10,
        ),
      );
      $form['image_panel']['more'] = array(
        '#type' => 'submit',
        '#value' => 'Add another image',
        '#ahah' => array(
          'path' => 'admin/build/promo_picker/images/ahah',
          'wrapper' => 'teaser-images',
          'method' => 'replace',
          'effect' => 'fade',
        ),
      );
    }
    else {
      for ($i = 0; $i < $num_images; $i++) {
        $form['image_panel']['images'][] = array(
          'image' => array(
            '#type' => 'textfield',
            '#title' => 'Image',
            '#autocomplete_path' => 'admin/build/promo_picker/image/autocomplete',
          ),
          'weight' => array(
            '#type' => 'weight',
            '#default_value' => 0,
            '#delta' => 10,
          ),
        );
      }
    }
  }
  // Add taxonomy fields.
  $form['taxonomy_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Taxonomy',
  );
  $taxonomies = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($taxonomies as $vid => $tids) {
    $vname = db_result(db_query('SELECT name FROM {vocabulary} WHERE vid = %d', $vid));
    $form['taxonomy_panel']['vocabulary_' . $vid] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($vname),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $options = array();
    foreach ($tids as $tid) {
      $tname = db_result(db_query('SELECT name FROM {term_data} WHERE tid = %d', $tid));
      if (!empty($tname)) {
        $options[$tid] = $tname;
      }
    }
    $form['taxonomy_panel']['vocabulary_' . $vid]['taxonomy_' . $vid] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => ((!empty($arg_tid) && in_array($arg_tid, $tids)) ? array($arg_tid) : array()),
    );
  }
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save teaser'),
  );
  return $form;
}

/**
 * Validate teaser creation form submission.
 *
 * @see node_promo_picker_teasers_admin_create_form()
 */
function node_promo_picker_teasers_admin_create_form_validate($form, &$state) {
  if (!empty($state['ahah_submission'])) {
    return;
  }
  $post = $state['values'];
  // Validate teaser title.
  if (empty($post['title'])) {
    form_set_error('title', 'No teaser title entered.');
  }
  // Validate teaser text.
  if (empty($post['teaser'])) {
    form_set_error('teaser', 'No teaser text entered.');
  }
  // Validate image nodes if selected.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $i) {
      if (!empty($i['image'])) {
        if (preg_match('/\([0-9]+\)$/', $i['image'], $match)) {
          $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          $exists = db_result(db_query('SELECT nid FROM {node} WHERE nid = %d', $nid));
          if (!$exists) {
            form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
          }
        }
        else {
          form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
        }
      }
    }
  }
}

/**
 * Process teaser creation form submission.
 *
 * @see node_promo_picker_teasers_admin_create_form()
 */
function node_promo_picker_teasers_admin_create_form_submit($form, &$state) {
  if (!empty($state['ahah_submission'])) {
    return;
  }
  $nid = arg(4);
  $post = $state['values'];
  // Create teaser record.
  $teaser = new stdClass();
  $teaser->title = $post['title'];
  $teaser->body = $post['teaser'];
  $teaser->related_nid = $nid;
  $teaser->primary_flag = (!empty($post['block']['primary'])) ? 1 : 0;
  $teaser->secondary_flag = (!empty($post['block']['secondary'])) ? 1 : 0;
  $teaser->uid = $GLOBALS['user']->uid;
  drupal_write_record('promo_picker_teasers', $teaser);
  // Create taxonomy records.
  $taxonomies = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($taxonomies as $vid => $tids) {
    if (!empty($post['taxonomy_' . $vid])) {
      foreach ($post['taxonomy_' . $vid] as $tid) {
        if ($tid) {
          // Get taxonomy name.
          $tname = db_result(db_query('SELECT name FROM {term_data} WHERE tid = %d', $tid));
          // Check if max teasers assigned to taxonomy.
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = %d AND ppt.primary_flag = 1';
          $count = db_result(db_query($sql, $tid));
          if ($count >= variable_get('promo_picker_max_primary_teasers', 3) && $teaser->primary_flag) {
            $teaser->primary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname primary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} AS ppt JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = %d AND ppt.secondary_flag = 1';
          $count = db_result(db_query($sql, $tid));
          if ($count >= variable_get('promo_picker_max_secondary_teasers', 3) && $teaser->secondary_flag) {
            $teaser->secondary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname secondary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          // Ensure node only has 1 teaser per taxonomy, per block.
          if ($teaser->primary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = %d AND pptt.tid = %d AND ppt.primary_flag = 1';
            db_query($sql, $teaser->related_nid, $tid);
          }
          if ($teaser->secondary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = %d AND pptt.tid = %d AND ppt.secondary_flag = 1';
            db_query($sql, $teaser->related_nid, $tid);
          }
          // Create taxonomy record.
          $tax = new stdClass();
          $tax->teaser_id = $teaser->teaser_id;
          $tax->tid = $tid;
          $tax->primary_delta = 0;
          $tax->secondary_delta = 0;
          drupal_write_record('promo_picker_teasers_taxonomy', $tax);
        }
      }
    }
  }
  // Create image records.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $delta => $img) {
      if (!empty($img['image'])) {
        if (preg_match('/\([0-9]+\)$/', $img['image'], $match)) {
          $inid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          // Update/insert image record.
          $image = new stdClass();
          $image->teaser_id = $teaser->teaser_id;
          $image->image_nid = $inid;
          $image->delta = $img['weight'];
          if (db_result(db_query('SELECT teaser_id FROM {promo_picker_teasers_images} WHERE teaser_id = %d AND image_nid = %d', $teaser->teaser_id, $inid))) {
            // Do nothing, image already added.
          }
          else {
            drupal_write_record('promo_picker_teasers_images', $image);
          }
        }
      }
    }
  }
  // Clear form cache.
  cache_clear_all('form_' . $state['values']['form_build_id'], 'cache_form');
  cache_clear_all('storage_' . $state['values']['form_build_id'], 'cache_form');
  // Set page to return to.
  drupal_set_message(t('Teaser record created.'));
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/build/promo_picker/add-teaser';
  $query = (!empty($_GET['referer'])) ? 'return_to=' . $_GET['referer'] : '';
  drupal_goto($goto, $query);
}

/**
 * Form definition for edit teaser record.
 *
 * @see node_promo_picker_teasers_admin_edit_form_validate()
 * @see node_promo_picker_teasers_admin_edit_form_submit()
 */
function node_promo_picker_teasers_admin_edit_form($state) {
  $form = array();
  // Get teaser record.
  $teaser_id = arg(4);
  $record = db_fetch_object(db_query('SELECT * FROM {promo_picker_teasers} WHERE teaser_id = %d', $teaser_id));
  if (empty($record)) {
    drupal_set_message(t('Invalid teaser record - @teaserid', array('@teaserid' => $teaser_id)), 'error');
    return $form;
  }
  // Get node.
  $node = db_fetch_object(db_query('SELECT n.*, t.name FROM {node} AS n JOIN {node_type} AS t ON n.type = t.type WHERE n.nid = %d', $record->related_nid));
  // Add node details.
  $form['node_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Node',
  );
  $form['node_panel']['node_id'] = array(
    '#type' => 'item',
    '#title' => 'NID',
    '#value' => $node->nid,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['node_panel']['node_type'] = array(
    '#type' => 'item',
    '#title' => 'Type',
    '#value' => $node->name,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  $form['node_panel']['node_title'] = array(
    '#type' => 'item',
    '#title' => 'Title',
    '#value' => $node->title,
    '#prefix' => '<div class="container-inline">',
    '#suffix' => '</div>',
  );
  // Add teaser fields.
  $form['teaser_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Teaser',
  );
  $form['teaser_panel']['title'] = array(
    '#type' => 'textfield',
    '#title' => 'Title',
    '#default_value' => $record->title,
  );
  $form['teaser_panel']['teaser'] = array(
    '#type' => 'textarea',
    '#title' => 'Teaser',
    '#default_value' => $record->body,
  );
  $block = array();
  if ($record->primary_flag) {
    $block[] = 'primary';
  }
  if ($record->secondary_flag) {
    $block[] = 'secondary';
  }
  $form['teaser_panel']['block'] = array(
    '#type' => 'checkboxes',
    '#title' => 'Block',
    '#description' => 'Specify promo block to display teaser in.',
    '#options' => array('primary' => 'Primary', 'secondary' => 'Secondary'),
    '#default_value' => $block,
  );
  $image_node = variable_get('promo_picker_image_node', NULL);
  if (!empty($image_node)) {
    // Get image nodes for teaser.
    $images = array();
    $result = db_query('SELECT n.title, ti.image_nid, ti.delta FROM {node} AS n JOIN {promo_picker_teasers_images} AS ti ON n.nid = ti.image_nid WHERE ti.teaser_id = %d ORDER BY ti.delta ASC', $teaser_id);
    while ($img = db_fetch_object($result)) {
      $images[] = $img;
    }
    // Add image fields.
    $form['image_panel'] = array(
      '#type' => 'fieldset',
      '#title' => 'Images',
    );
    $form['image_panel']['images'] = array(
      '#tree' => TRUE,
      '#prefix' => '<div id="teaser-images">',
      '#suffix' => '</div>',
      '#theme' => 'form_teaser_images',
    );
    $num_images = variable_get('promo_picker_max_images', 3);
    foreach ($images as $i) {
      $form['image_panel']['images'][] = array(
        'image' => array(
          '#type' => 'textfield',
          '#title' => '',
          '#autocomplete_path' => 'admin/build/promo_picker/image/autocomplete',
          '#default_value' => $i->title . ' (' . $i->image_nid . ')',
        ),
        'weight' => array(
          '#type' => 'weight',
          '#default_value' => $i->delta,
          '#delta' => 10,
        ),
      );
    }
    for ($i = count($images); $i < $num_images; $i++) {
      $form['image_panel']['images'][] = array(
        'image' => array(
          '#type' => 'textfield',
          '#title' => '',
          '#autocomplete_path' => 'admin/build/promo_picker/image/autocomplete',
        ),
        'weight' => array(
          '#type' => 'weight',
          '#default_value' => $i,
          '#delta' => 10,
        ),
      );
    }
    if ($num_images == 0) {
      if (count($images) < 1) {
        $form['image_panel']['images'][] = array(
          'image' => array(
            '#type' => 'textfield',
            '#title' => '',
            '#autocomplete_path' => 'admin/build/promo_picker/image/autocomplete',
          ),
          'weight' => array(
            '#type' => 'weight',
            '#default_value' => 0,
            '#delta' => 10,
          ),
        );
      }
      $form['image_panel']['more'] = array(
        '#type' => 'submit',
        '#value' => 'Add another image',
        '#ahah' => array(
          'path' => 'admin/build/promo_picker/images/ahah',
          'wrapper' => 'teaser-images',
          'method' => 'replace',
          'effect' => 'fade',
        ),
      );
    }
  }
  // Get teaser taxonomy association.
  $teaser_tids = array();
  $result = db_query('SELECT tid FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = %d', $teaser_id);
  while ($t = db_result($result)) {
    $teaser_tids[] = $t;
  }
  // Add taxonomy fields.
  $form['taxonomy_panel'] = array(
    '#type' => 'fieldset',
    '#title' => 'Taxonomy',
  );
  $taxonomies = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($taxonomies as $vid => $tids) {
    $vname = db_result(db_query('SELECT name FROM {vocabulary} WHERE vid = %d', $vid));
    $form['taxonomy_panel']['vocabulary_' . $vid] = array(
      '#type' => 'fieldset',
      '#title' => check_plain($vname),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $options = array();
    foreach ($tids as $tid) {
      $tname = db_result(db_query('SELECT name FROM {term_data} WHERE tid = %d', $tid));
      if (!empty($tname)) {
        $options[$tid] = $tname;
      }
    }
    $form['taxonomy_panel']['vocabulary_' . $vid]['taxonomy_' . $vid] = array(
      '#type' => 'checkboxes',
      '#options' => $options,
      '#default_value' => $teaser_tids,
    );
  }
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Save teaser'),
  );
  return $form;
}

/**
 * Validate teaser edit form submission.
 *
 * @see node_promo_picker_teasers_admin_edit_form()
 */
function node_promo_picker_teasers_admin_edit_form_validate($form, &$state) {
  if (!empty($state['ahah_submission'])) {
    return;
  }
  $post = $state['values'];
  // Validate teaser title.
  if (empty($post['title'])) {
    form_set_error('title', 'No teaser title entered.');
  }
  // Validate teaser text.
  if (empty($post['teaser'])) {
    form_set_error('teaser', 'No teaser text entered.');
  }
  // Validate image nodes if selected.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $i) {
      if (!empty($i['image'])) {
        if (preg_match('/\([0-9]+\)$/', $i['image'], $match)) {
          $nid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          $exists = db_result(db_query('SELECT nid FROM {node} WHERE nid = %d', $nid));
          if (!$exists) {
            form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
          }
        }
        else {
          form_set_error('image', 'Invalid image selected - ' . $i['image'] . '.');
        }
      }
    }
  }
}

/**
 * Process teaser creation form submission.
 *
 * @see node_promo_picker_teasers_admin_edit_form()
 */
function node_promo_picker_teasers_admin_edit_form_submit($form, &$state) {
  if (!empty($state['ahah_submission'])) {
    return;
  }
  $teaser_id = arg(4);
  $post = $state['values'];
  // Create teaser record.
  $teaser = new stdClass();
  $teaser->teaser_id = $teaser_id;
  $teaser->title = $post['title'];
  $teaser->body = $post['teaser'];
  $teaser->primary_flag = (!empty($post['block']['primary'])) ? 1 : 0;
  $teaser->secondary_flag = (!empty($post['block']['secondary'])) ? 1 : 0;
  drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
  // Get teaser taxonomy records, to check if any need deleting.
  $result = db_query('SELECT tid FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = %d', $teaser->teaser_id);
  $teaser_tids = array();
  while ($row = db_result($result)) {
    $teaser_tids[$row] = $row;
  }
  // Create taxonomy records.
  $taxonomies = variable_get('promo_picker_taxonomy_settings', array());
  foreach ($taxonomies as $vid => $tids) {
    if (!empty($post['taxonomy_' . $vid])) {
      foreach ($post['taxonomy_' . $vid] as $tid) {
        if ($tid) {
          // Remove tid from array so it is not deleted.
          if (!empty($teaser_tids[$tid])) {
            unset($teaser_tids[$tid]);
          }
          // Check if taxonomy record already exists, no need to update it.
          if (db_result(db_query('SELECT teaser_id FROM {promo_picker_teasers_taxonomy} WHERE teaser_id = %d AND tid = %d', $teaser->teaser_id, $tid))) {
            continue;
          }
          // Get taxonomy name.
          $tname = db_result(db_query('SELECT name FROM {term_data} WHERE tid = %d', $tid));
          // Check if max teasers assigned to taxonomy.
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = %d AND ppt.primary_flag = 1';
          $count = db_result(db_query($sql, $tid));
          if ($count >= variable_get('promo_picker_max_primary_teasers', 3) && $teaser->primary_flag) {
            $teaser->primary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname primary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          $sql = 'SELECT COUNT(ppt.teaser_id) FROM {promo_picker_teasers} JOIN {promo_picker_teasers_taxonomy} AS pptt ON ppt.teaser_id = pptt.teaser_id WHERE pptt.tid = %d AND ppt.secondary_flag = 1';
          $count = db_result(db_query($sql, $tid));
          if ($count >= variable_get('promo_picker_max_secondary_teasers', 3) && $teaser->secondary_flag) {
            $teaser->secondary_flag = 0;
            drupal_write_record('promo_picker_teasers', $teaser, 'teaser_id');
            drupal_set_message(t('Could not add teaser to @tname secondary block as maximum teaser limit reached.', array('@tname' => $tname)), 'warning');
            continue;
          }
          // Ensure node only has 1 teaser per taxonomy, per block.
          if ($teaser->primary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = %d AND pptt.tid = %d AND ppt.primary_flag = 1';
            db_query($sql, $teaser->related_nid, $tid);
          }
          if ($teaser->secondary_flag) {
            $sql = 'DELETE pptt FROM {promo_picker_teasers_taxonomy} AS pptt JOIN {promo_picker_teasers} AS ppt ON pptt.teaser_id = ppt.teaser_id WHERE ppt.related_nid = %d AND pptt.tid = %d AND ppt.secondary_flag = 1';
            db_query($sql, $teaser->related_nid, $tid);
          }
          // Create taxonomy record.
          $tax = new stdClass();
          $tax->teaser_id = $teaser->teaser_id;
          $tax->tid = $tid;
          $tax->primary_delta = 0;
          $tax->secondary_delta = 0;
          drupal_write_record('promo_picker_teasers_taxonomy', $tax);
        }
      }
    }
  }
  // Delete any removed taxonomies.
  if (!empty($teaser_tids)) {
    foreach ($teaser_tids as $rtid) {
      db_query('DELETE FROM {promo_picker_teasers_taxonomy} WHERE tid = %d AND teaser_id = %d', $rtid, $teaser->teaser_id);
    }
  }
  // Create image records.
  if (!empty($post['images'])) {
    foreach ($post['images'] as $img) {
      if (!empty($img['image'])) {
        if (preg_match('/\([0-9]+\)$/', $img['image'], $match)) {
          $inid = (int) preg_replace('/(\(|\))/', '', $match[0]);
          // Update/insert image record.
          $image = new stdClass();
          $image->teaser_id = $teaser->teaser_id;
          $image->image_nid = $inid;
          $image->delta = $img['weight'];
          if (db_result(db_query('SELECT teaser_id FROM {promo_picker_teasers_images} WHERE teaser_id = %d AND image_nid = %d', $teaser->teaser_id, $inid))) {
            drupal_write_record('promo_picker_teasers_images', $image, array('teaser_id', 'image_nid'));
          }
          else {
            drupal_write_record('promo_picker_teasers_images', $image);
          }
        }
      }
    }
  }
  // Clear form cache.
  cache_clear_all('form_' . $state['values']['form_build_id'], 'cache_form');
  cache_clear_all('storage_' . $state['values']['form_build_id'], 'cache_form');
  // Set page to return to.
  drupal_set_message(t('Teaser record updated.'));
  $goto = (!empty($_GET['return_to'])) ? $_GET['return_to'] : 'admin/build/promo_picker/teasers';
  $query = (!empty($_GET['referer'])) ? 'return_to=' . $_GET['referer'] : '';
  drupal_goto($goto, $query);
}

/**
 * Build json response to add more image fields.
 */
function node_promo_picker_ahah_image_field() {
  $form_state = array(
    'storage' => NULL,
    'submitted' => FALSE,
    'rebuild' => TRUE,
  );
  $form_build_id = check_plain($_POST['form_build_id']);
  $form = form_get_cache($form_build_id, $form_state);
  $args = $form['#parameters'];
  $form_id = array_shift($args);
  $form_state['post'] = $form['#post'] = $_POST;
  // Enable the submit/validate handlers to determine whether AHAH-submittted.
  $form_state['ahah_submission'] = TRUE;
  $form['#programmed'] = $form['#redirect'] = FALSE;
  // Add extra image field, if max not reached.
  $form['image_panel']['images'][] = array(
    'image' => array(
      '#type' => 'textfield',
      '#title' => '',
      '#autocomplete_path' => 'admin/build/promo_picker/image/autocomplete',
    ),
    'weight' => array(
      '#type' => 'weight',
      '#default_value' => 0,
      '#delta' => 10,
    ),
  );
  drupal_process_form($form_id, $form, $form_state);
  // Save form back into cache.
  form_set_cache($form_build_id, $form, $form_state);
  // Get images element to render.
  $images = $form['image_panel']['images'];
  // Remove the prefix/suffix wrapper so we don't double it up.
  unset($images['#prefix'], $images['#suffix']);
  // Render the output.
  $output = theme('status_messages');
  $output .= drupal_render($images);
  // Rendering callback.
  drupal_json(array('status' => TRUE, 'data' => $output));
  exit();
}

/**
 * Theme the images field in teaser creation/edit forms.
 */
function theme_form_teaser_images($element) {
  $html = '';
  foreach (element_children($element) as $key) {
    if (isset($element[$key]['image'])) {
      $image = &$element[$key];
      $row = array();
      $row[] = drupal_render($image['image']);
      $image['weight']['#attributes']['class'] = 'image-weight';
      $row[] = drupal_render($image['weight']);
      $rows[] = array('data' => $row, 'class' => 'draggable');
    }
  }
  $header = array('Image', 'Weight');
  drupal_add_tabledrag('image-list', 'order', 'sibling', 'image-weight');
  return theme('table', $header, $rows, array('id' => 'image-list'));
}

/**
 * Theme the teaser list table.
 */
function theme_teaser_list_table($element) {
  if (!empty($element['rows']['#value'])) {
    $rows = $element['rows']['#value'];
  }
  else {
    $rows[] = array(
      array(
        'data' => t('No posts available.'),
        'colspan' => '7',
      ),
    );
  }
  $output .= theme('table', $element['header']['#value'], $rows);
  if ($element['pager']['#value']) {
    $output .= drupal_render($element['pager']);
  }
  return $output;
}
